/*
 *    NzdlCorbaFactory.java
 *    Copyright (C) 2001 New Zealand Digital Library Project
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

//the package we're in
package org.nzdl.gsdl.util;

import java.util.Set;
import java.util.HashSet;
import java.util.List;
import java.util.ListIterator;

import org.nzdl.gsdl.service.NzdlQuery;
import org.nzdl.gsdl.service.NzdlBrowse;
import org.nzdl.gsdl.corba.gsdlInterface.*;
import org.nzdl.gsdl.util.NzdlConstants;

/**
 * Returns CORBA objects via it's operations.
 */
public final class NzdlCorbaFactory {


  private NzdlCorbaFactory() {}

  public static corbatext_t toCorbaText(String _text) {
    int len = _text.length();
    short[] shorts = new short[len];
    for (int i=0; i<len; i++) {
      shorts[i] = (short)_text.charAt(i);
    }
    short encoding = 0;
    corbatext_t corba_t = new corbatext_t(shorts, encoding, len);
    return corba_t;
  }

  public static String toString(corbatext_t _text) {
    //    System.out.println("NzdlCorbaFactory.toString() called length = " + _text.text.length +
    //		       " ecnoding = " + _text.encoding +
    //		       " reported length = " + _text.length);
    StringBuffer text = new StringBuffer(1000);
    for (int j=0; j<_text.text.length; j++) {
      text.append((char)_text.text[j]);
    }
    return text.toString();
  }

  public static corbaComErrorHolder createComErrorHolder() {
    corbaComErrorHolder err
      = new corbaComErrorHolder( corbaComError.from_int(0) );    
    return err;
  }

  public static corbatext_tarrayHolder createCorbaTextArrayHolder() {
    corbatext_t [] array = {};
    corbatext_tarrayHolder arrayHolder = new corbatext_tarrayHolder( array );
    return arrayHolder;
  }

  public static Set  toSet( corbatext_t [] _configLines ) {
    HashSet set = new HashSet();
      for (int i=0; i<_configLines.length; i++) {
	set.add(NzdlCorbaFactory.toString(_configLines[i]));
      }
      return set;
  }

  public static corbatext_t []  toCorbaTextArray( Set _configLines ) {
    Object [] lines = _configLines.toArray();
    corbatext_t [] array = new corbatext_t[ lines.length ];
    for (int i=0; i<lines.length; i++) {
      array[i] = toCorbaText( (String)lines[i] );
    }
    return array;
  }

  public static corbaColInfoResponseHolder createColInfoResponseHolder() {
    corbaShortColInfo shortInfo = new corbaShortColInfo();
    shortInfo.name = toCorbaText("");
    shortInfo.host = toCorbaText("");
    corbaColInfoResponse response = new corbaColInfoResponse();
    response.shoftInfo = shortInfo; // shoft !! typo in corba interface !!
    response.isPublic  = false;
    response.isBeta    = false;
    response.buildDate = 0;
    response.ccsCols   = new corbatext_t[0];
    response.languages = new corbatext_t[0];
    response.numDocs   = 0;
    response.numWords  = 0;
    response.numBytes  = 0;
    response.collectionMeta = new corbatext_tmap(new corbatext_t[0],new corbatext_t[0]);
    response.format         = new corbatext_tmap(new corbatext_t[0],new corbatext_t[0]);
    response.building       = new corbatext_tmap(new corbatext_t[0],new corbatext_t[0]);
    response.httpdomain   = toCorbaText("");
    response.httpprefix   = toCorbaText("");
    response.receptionist = toCorbaText("");
    corbaColInfoResponseHolder responseHolder 
      = new corbaColInfoResponseHolder(response);
    return responseHolder;
  }
    
  public static corbaFilterRequest 
    createQueryFilterRequest( String _term, String _type, int _numOfHits ) {
    return createQueryFilterRequest( _term,
				     _type,
				     String.valueOf(NzdlConstants.DEFAULT_CASE_FOLDING),
				     String.valueOf(NzdlConstants.DEFAULT_STEMMING),
				     String.valueOf(NzdlConstants.DEFAULT_START_RESULTS),
				     String.valueOf(_numOfHits),
				     String.valueOf(NzdlConstants.DEFAULT_MAX_DOCS),
				     NzdlConstants.DEFAULT_QUERY_META,
				     new String("")
				     );
  }

  public static corbaFilterRequest 
    createQueryFilterRequest( NzdlQuery _query ) {
	return createQueryFilterRequest( _query.getQueryTerm(),
				     _query.getQueryType(),
				     _query.getCaseFolding(),
				     _query.getStemming(),
				     String.valueOf(_query.getStartResults()),
				     String.valueOf(_query.getEndResults()),
				     String.valueOf(_query.getMaxDocs()),
					 _query.getMetadata(),
					 _query.getIndex()
				     );
  }


  public static corbaFilterRequest 
    createQueryFilterRequest( String _term, String _type,     String _case, 
			      String _stem, String _startHit, String _endHit, String _maxDocs) {
		return createQueryFilterRequest(_term, _type, _case, _stem, _startHit, _endHit, _maxDocs,
		                                NzdlConstants.DEFAULT_QUERY_META, new String("")
						 );
	}

  public static corbaFilterRequest 
    createQueryFilterRequest( String _term, String _type,     String _case, 
			      String _stem, String _startHit, String _endHit, String _maxDocs, boolean _meta, String _index ) {

    corbaFilterRequest queryFilter = new corbaFilterRequest();
    queryFilter.filter        = toCorbaText("QueryFilter");

    queryFilter.filterOptions = new corbaOptionValue[9];
    queryFilter.filterOptions[0] 
      = new corbaOptionValue(toCorbaText("Term"),
			     toCorbaText(_term));
    queryFilter.filterOptions[1] 
      = new corbaOptionValue(toCorbaText("QueryType"),
			     toCorbaText(_type));
    queryFilter.filterOptions[2] 
      = new corbaOptionValue(toCorbaText("Casefold"),
			     toCorbaText(_case));
    queryFilter.filterOptions[3] 
      = new corbaOptionValue(toCorbaText("Stem"),
			     toCorbaText(_stem));
    queryFilter.filterOptions[4] 
      = new corbaOptionValue(toCorbaText("StartResults"),
			     toCorbaText(_startHit));
    queryFilter.filterOptions[5] 
      = new corbaOptionValue(toCorbaText("EndResults"),
			     toCorbaText(_endHit));
    queryFilter.filterOptions[6] 
      = new corbaOptionValue(toCorbaText("Maxdocs"),
			     toCorbaText(_maxDocs));

      queryFilter.filterOptions[7] 
        = new corbaOptionValue(toCorbaText("MatchMode"),
  			     toCorbaText("all"));
      queryFilter.filterOptions[8]
	  = new corbaOptionValue(toCorbaText("Index"),
				 toCorbaText(_index));

    queryFilter.docSet        = new corbatext_t[0];
    int resultOptions = NzdlConstants.FROID|NzdlConstants.FRtermFreq|NzdlConstants.FRranking;
		if (_meta)
		{	resultOptions |= NzdlConstants.FRmetadata;
		}
    queryFilter.filterResultOptions = resultOptions;
    queryFilter.requestParams = toCorbaText("");
    queryFilter.refParams     = toCorbaText("");
    queryFilter.fields        = new corbatext_t[0];
    queryFilter.getParents    = false;
    return queryFilter;
  }
  
  public static corbaFilterRequest 
    createBrowseFilterRequest( NzdlBrowse _browse ) {
    return createBrowseFilterRequest( _browse.getBrowseNode(), _browse.getMetadata());
  }

  public static corbaFilterRequest createBrowseFilterRequest(String _docID, String _metaTag) {

    corbaFilterRequest browseFilter = new corbaFilterRequest();
    browseFilter.filter        = toCorbaText("BrowseFilter");

    browseFilter.filterOptions = new corbaOptionValue[1];
    browseFilter.filterOptions[0] 
      = new corbaOptionValue(toCorbaText("ParentNode"),
			     toCorbaText(_docID));

    browseFilter.docSet        = new corbatext_t[0];
    int resultOptions = NzdlConstants.FROID|NzdlConstants.FRmetadata;
    browseFilter.filterResultOptions = resultOptions;
    browseFilter.requestParams = toCorbaText("");
    browseFilter.refParams     = toCorbaText("");
    browseFilter.fields        = new corbatext_t[1];
    browseFilter.fields[0]     = toCorbaText(_metaTag);
    browseFilter.getParents    = false;
    return browseFilter;
  }

  public static corbaFilterRequest createBrowseFilterRequest( String _docID ) {
    corbaFilterRequest browseFilter = new corbaFilterRequest();
    browseFilter.filter        = toCorbaText("BrowseFilter");
    browseFilter.filterOptions = new corbaOptionValue[1];
    browseFilter.filterOptions[0] 
      = new corbaOptionValue(toCorbaText("ParentNode"),
			     toCorbaText(_docID));
    browseFilter.docSet        = new corbatext_t[0];
    int resultOptions = NzdlConstants.FROID;
    browseFilter.filterResultOptions = resultOptions;
    browseFilter.requestParams = toCorbaText("");
    browseFilter.refParams     = toCorbaText("");
    browseFilter.fields        = new corbatext_t[0];
    browseFilter.getParents    = false;
    return browseFilter;
  }

  public static corbaFilterRequest createMetaDataFilterRequest( String _docID, String _metaTag ) {
    corbaFilterRequest nullFilter = new corbaFilterRequest();
    nullFilter.filter        = toCorbaText("NullFilter");
    nullFilter.filterOptions = new corbaOptionValue[0];
    nullFilter.docSet        = new corbatext_t[1];
    nullFilter.docSet[0]     = toCorbaText(_docID);
    int resultOptions = NzdlConstants.FRmetadata;
    nullFilter.filterResultOptions = resultOptions;
    nullFilter.requestParams = toCorbaText("");
    nullFilter.refParams     = toCorbaText("");
    nullFilter.fields        = new corbatext_t[1];
    nullFilter.fields[0]     = toCorbaText(_metaTag);
    nullFilter.getParents    = false;
    return nullFilter;
  }

  public static corbaFilterRequest createMetaDataFilterRequest( List _docIDs, String _metaTag ) {
    corbaFilterRequest nullFilter = new corbaFilterRequest();
    nullFilter.filter = toCorbaText("NullFilter");
    nullFilter.filterOptions = new corbaOptionValue[0];
    // we will request the meta data for all documents in _docIDs set
    nullFilter.docSet = new corbatext_t[_docIDs.size()];
    int i=0;
    for (ListIterator itr = _docIDs.listIterator(); itr.hasNext(); ) {
      nullFilter.docSet[i] = toCorbaText( (String)itr.next() );
      i++;
    }
    int resultOptions = NzdlConstants.FRmetadata;
    nullFilter.filterResultOptions = resultOptions;
    nullFilter.requestParams = toCorbaText("");
    nullFilter.refParams     = toCorbaText("");
    nullFilter.fields        = new corbatext_t[1];
    // this is the meta (_metaTag) data we need
    nullFilter.fields[0]     = toCorbaText(_metaTag);
    nullFilter.getParents    = false;
    return nullFilter;
  }

  public static corbaFilterResponseHolder createFilterResponseHolder() {
    corbaFilterResponse responseFilter = new corbaFilterResponse();
    responseFilter.numDocs = 0;
    responseFilter.isApprox = corbaIsApprox.from_int(0);
    responseFilter.termInfo = new corbaTermInfo[0];
    responseFilter.docInfo = new corbaResultDocInfo[0];
    return new corbaFilterResponseHolder(responseFilter);
  }

    // created 25-4-02 kjdon
    public static corbaFilterOptionsResponseHolder createFilterOptionsResponseHolder() {
	corbaFilterOptionsResponse response = new corbaFilterOptionsResponse();
	response.options = new corbaFilterOption[0];
	response.names = new corbatext_t[0];
	return new corbaFilterOptionsResponseHolder(response);
    }

  public static corbaDocRequestHolder createDocRequestHolder( String _docID ) {
    corbaDocRequest docRequest 
      = new corbaDocRequest( toCorbaText( _docID ), toCorbaText(""), toCorbaText("") );
    return new corbaDocRequestHolder(docRequest);
  }

  public static corbaDocResponseHolder createDocResponseHolder( ) {
    corbaDocResponse docResponse = new corbaDocResponse( toCorbaText("") );
    return new corbaDocResponseHolder( docResponse );
  }

}





